#include "ccvt.h"

#include <stdio.h>

/* #define VIDEO_PALETTE_GREY	1	/\* Linear greyscale *\/ */
/* #define VIDEO_PALETTE_HI240	2	/\* High 240 cube (BT848) *\/ */
/* #define VIDEO_PALETTE_RGB565	3	/\* 565 16 bit RGB *\/ */
/* #define VIDEO_PALETTE_RGB24	4	/\* 24bit RGB *\/ */
/* #define VIDEO_PALETTE_RGB32	5	/\* 32bit RGB *\/	 */
/* #define VIDEO_PALETTE_RGB555	6	/\* 555 15bit RGB *\/ */
/* #define VIDEO_PALETTE_YUV422	7	/\* YUV422 capture *\/ */
/* #define VIDEO_PALETTE_YUYV	8 */
/* #define VIDEO_PALETTE_UYVY	9	/\* The great thing about standards is ... *\/ */
/* #define VIDEO_PALETTE_YUV420	10 */
/* #define VIDEO_PALETTE_YUV411	11	/\* YUV411 capture *\/ */
/* #define VIDEO_PALETTE_RAW	12	/\* RAW capture (BT848) *\/ */
/* #define VIDEO_PALETTE_YUV422P	13	/\* YUV 4:2:2 Planar *\/ */
/* #define VIDEO_PALETTE_YUV411P	14	/\* YUV 4:1:1 Planar *\/ */
/* #define VIDEO_PALETTE_YUV420P	15	/\* YUV 4:2:0 Planar *\/ */
/* #define VIDEO_PALETTE_YUV410P	16	/\* YUV 4:1:0 Planar *\/ */
/* #define VIDEO_PALETTE_PLANAR	13	/\* start of planar entries *\/ */
/* #define VIDEO_PALETTE_COMPONENT 7	/\* start of component entries *\/ */

static char* palettes_names[] =
  {
    "GREY - Linear greyscale",
    "HI240 - High 240 cube (BT848)",
    "RGB565 - 565 16 bit RGB",
    "RGB24 - 24bit RGB",
    "RGB32 - 32bit RGB",
    "RGB555 - 555 15bit RGB",
    "YUV422 - YUV422 capture",
    "YUYV",
    "UYVY - The great thing about standards is ...",
    "YUV420",
    "YUV411 - YUV411 capture",
    "RAW - RAW capture (BT848)",
    "YUV422P - YUV 4:2:2 Planar",
    "YUV411P - YUV 4:1:1 Planar",
    "YUV420P - YUV 4:2:0 Planar",
    "YUV410P - YUV 4:1:0 Planar"
  };

static float palettes_bytes_per_pixel[] =
  {
    0,
    0,
    2,
    3,
    4,
    0,
    0,
    0,
    0,
    1.5,
    0,
    0,
    0,
    0,
    1.5,
    0
  };

void BGR24ToRGB32(int width, int height, const void *voidSrc, void *voidDst) {
   int i;

   unsigned char* dest = (unsigned char*) voidDst;
   unsigned char* src  = (unsigned char*) voidSrc;

   unsigned char red, green, blue, alpha;

   for (i = 0; i < width * height; i++) {
      blue  = *src++;
      green = *src++;
      red   = *src++;
      alpha = 255;

      *dest++ = red;
      *dest++ = green;
      *dest++ = blue;
      *dest++ = alpha;
   }
}

typedef void (*converter)(int width, int height, const void *src, void *dst);
static converter palettes_functions[] =
  {
    0,
    0,
    0,
    BGR24ToRGB32,
    0,
    0,
    0,
    0,
    0,
    ccvt_420p_rgb32,
    0,
    0,
    0,
    0,
    ccvt_420p_rgb32,
    0
  };



float paletteBytesPerPixel(int palette) {
   float bytesPerPixel;
   if (palette < 1 || palette > 16) {
      fprintf(stderr, "* Invalid palette=%d\n", palette);
      return 0;
   }

   bytesPerPixel = palettes_bytes_per_pixel[palette - 1];
   if (bytesPerPixel == 0) {
      fprintf(stderr, "* Palette=%d not yet supported\n", palette);
      return 0;
   }

   return bytesPerPixel;
}



char* paletteName(int palette) {
   if (palette == 0) {
      return "*automatic*";
   }
   if (palette < 0 || palette > 16) {
      return "*UNKOWN*";
   }
   
   return palettes_names[palette - 1];
}



int palleteConvert(int palette,
		   int width, int height,
		   const void *src, void *dst) {
   converter function;

   if (palette < 1 || palette > 16) {
      fprintf(stderr, "* Invalid palette=%d\n", palette);
      return 0;
   }

   function = palettes_functions[palette - 1];
   if (function == 0) {
      fprintf(stderr, "* Palette=%d not yet supported\n", palette);
      return 0;
   }

   /*fprintf(stderr, "* Converting palette=%d width=%d height=%d \n", palette, width, height);*/

   function(width, height, src, dst);

   return 1;
}

